/*
 * TclDotNetForm.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using System;
using System.Drawing;
using System.Reflection;
using System.Windows.Forms;
using Eagle._Components.Public;
using Eagle._Components.Public.Delegates;
using Eagle._Containers.Public;
using Eagle._Interfaces.Public;
using System.Threading;

namespace TclDotNet.Tutorial
{
    public partial class TclDotNetForm : Form
    {
        #region Private Data
        private Interpreter interpreter;
        private bool owner;
        private bool disposed;
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Private Delegate Types
        private delegate void DelegateWithNoArgs();
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Private Constructors
        private TclDotNetForm(
            Interpreter interpreter,
            bool owner
            )
        {
            InitializeComponent();

            ///////////////////////////////////////////////////////////////////

            this.Text = Application.ProductName;

            ///////////////////////////////////////////////////////////////////

            #region Event Setup
            this.FormClosing +=
                new FormClosingEventHandler(TclDotNetForm_FormClosing);

            this.Disposed +=
                new System.EventHandler(TclDotNetForm_Disposed);
            #endregion

            ///////////////////////////////////////////////////////////////////

            #region Private Data Setup
            //
            // NOTE: Setup our private data.
            //
            this.interpreter = interpreter;
            this.owner = owner;
            #endregion

            ///////////////////////////////////////////////////////////////////

            #region Custom Command Setup
            long token = 0;
            Result result = null;

            if ((interpreter != null) && !interpreter.Disposed &&
                interpreter.AddExecuteCallback(
                    "changeFormColor", ChangeFormColorCommandCallback,
                    null, ref token, ref result) != ReturnCode.Ok)
            {
                ShowResult(ReturnCode.Error, result);
            }
            #endregion
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Private Methods
        private static void ShowResult(
            ReturnCode code,
            Result result
            )
        {
            MessageBox.Show(
                Utility.FormatResult(code, result),
                Application.ProductName, MessageBoxButtons.OK,
                Utility.IsSuccess(code, false) ?
                MessageBoxIcon.Information : MessageBoxIcon.Error);
        }

        ////////////////////////////////////////////////////////////////////////

        private ReturnCode CancelOrComplain()
        {
            ReturnCode code;
            Result error = null;

            if ((interpreter != null) && !interpreter.Disposed)
            {
                //
                // NOTE: Attempt to cancel any scripts in progress.
                //
                code = interpreter.CancelEvaluate(
                    null, CancelFlags.UnwindAndNotify, ref error);
            }
            else
            {
                error = "invalid or disposed interpreter";
                code = ReturnCode.Error;
            }

            if (code != ReturnCode.Ok)
            {
                //
                // NOTE: We failed to cancel the scripts, display the error
                //       and prevent the form from being closed.
                //
                ShowResult(code, error);
            }

            return code;
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Form Event Handlers
        private void TclDotNetForm_FormClosing(
            object sender,
            FormClosingEventArgs e
            )
        {
            CancelOrComplain();
        }

        ///////////////////////////////////////////////////////////////////////

        private void TclDotNetForm_Disposed(
            object sender,
            EventArgs e
            )
        {
            if (!disposed)
            {
                if (owner && (interpreter != null))
                {
                    interpreter.Dispose();
                    interpreter = null;
                }

                disposed = true;
            }
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Button Event Handlers
        private void btnEvaluate_Click(
            object sender,
            EventArgs e
            )
        {
            ReturnCode code;
            Result result = null;

            if ((interpreter != null) && !interpreter.Disposed)
            {
                code = interpreter.EvaluateScript(
                    this.txtScript.Text, AsyncEvaluateCallback,
                    null, ref result);
            }
            else
            {
                result = "invalid or disposed interpreter";
                code = ReturnCode.Error;
            }

            if (code == ReturnCode.Ok)
                this.Text = String.Format(
                    "{0} - busy...", Application.ProductName);
            else
                ShowResult(code, result);
        }

        ///////////////////////////////////////////////////////////////////////

        private void btnCancel_Click(
            object sender,
            EventArgs e
            )
        {
            CancelOrComplain();
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Delegate Command Implementations
        private ReturnCode ChangeFormColorCommandCallback(
            Interpreter interpreter,
            IClientData clientData,
            ArgumentList arguments,
            ref Result result
            )
        {
            ReturnCode code = ReturnCode.Ok;
            Color color = Color.Aqua;

            if ((arguments != null) && (arguments.Count > 1))
            {
                try
                {
                    PropertyInfo propertyInfo = typeof(Color).GetProperty(
                        arguments[1], BindingFlags.IgnoreCase |
                        BindingFlags.Static | BindingFlags.Public);

                    if (propertyInfo != null)
                    {
                        color = (Color)propertyInfo.GetValue(null, null);
                    }
                    else
                    {
                        result = String.Format(
                            "invalid color \"{0}\"",
                            arguments[1]);

                        code = ReturnCode.Error;
                    }
                }
                catch (Exception e)
                {
                    result = e;
                    code = ReturnCode.Error;
                }
            }

            if (code == ReturnCode.Ok)
            {
                if (this.InvokeRequired)
                {
                    BeginInvoke(new DelegateWithNoArgs(delegate()
                    {
                        this.BackColor = color;
                    }));
                }
                else
                {
                    this.BackColor = color;
                }

                result = color.ToString();
            }

            return code;
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Asynchronous Evaluation Callbacks
        private void AsyncEvaluateCallback(
            IAsynchronousContext context
            )
        {
            BeginInvoke(
                new AsynchronousCallback(AsynchronousCallback),
                new object[] { context });
        }

        ///////////////////////////////////////////////////////////////////////

        private void AsynchronousCallback(
            IAsynchronousContext context
            )
        {
            ReturnCode code;
            Result result;

            if (context != null)
            {
                result = context.Result;
                code = context.ReturnCode;
            }
            else
            {
                result = "invalid context";
                code = ReturnCode.Error;
            }

            if (Utility.IsSuccess(code, false))
                this.txtResult.BackColor = Color.Green;
            else
                this.txtResult.BackColor = Color.Red;

            this.txtResult.Text = Utility.FormatResult(code, result);
            this.Text = Application.ProductName;
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Static "Factory" Methods
        [STAThread()]
        public static ReturnCode Show(
            Interpreter interpreter,
            bool owner,
            bool start,
            string[] args,
            ref Thread thread,
            ref Result error
            )
        {
            try
            {
                thread = new Thread(new ThreadStart(delegate()
                {
                    TclDotNetForm form =
                        new TclDotNetForm(interpreter, owner);

                    form.ShowDialog();
                }));

                if (start)
                    thread.Start();

                return ReturnCode.Ok;
            }
            catch (Exception e)
            {
                error = e;
            }

            return ReturnCode.Error;
        }
        #endregion
    }
}
